@@ -18,18 +18,17 @@ the Characteristic does not block waiting for the actual send to happen.
 import asyncio
 import base64
 from concurrent.futures import ThreadPoolExecutor
-import functools
 import hashlib
-import tempfile
-import json
 import logging
 import os
+import re
 import socket
 import sys
+import tempfile
 import threading
-import re
 
-from zeroconf import ServiceInfo, Zeroconf
+from zeroconf import ServiceInfo
+from zeroconf.asyncio import AsyncZeroconf
 
 from pyhap import util
 from pyhap.accessory import get_topic
@@ -42,6 +41,7 @@ from pyhap.const import (
     HAP_REPR_IID,
     HAP_REPR_STATUS,
     HAP_REPR_VALUE,
+    MAX_CONFIG_VERSION,
     STANDALONE_AID,
 )
 from pyhap.encoder import AccessoryEncoder
@@ -50,12 +50,12 @@ from pyhap.hsrp import Server as SrpServer
 from pyhap.loader import Loader
 from pyhap.params import get_srp_context
 from pyhap.state import State
+
+from .const import HAP_SERVER_STATUS
 from .util import callback
 
 logger = logging.getLogger(__name__)
 
-CHAR_STAT_OK = 0
-SERVICE_COMMUNICATION_FAILURE = -70402
 SERVICE_CALLBACK = "callback"
 SERVICE_CHARS = "chars"
 SERVICE_IIDS = "iids"
@@ -63,20 +63,6 @@ HAP_SERVICE_TYPE = "_hap._tcp.local."
 VALID_MDNS_REGEX = re.compile(r"[^A-Za-z0-9\-]+")
 
 
-def is_callback(func):
-    """Check if function is callback."""
-    return "_pyhap_callback" in getattr(func, "__dict__", {})
-
-
-def iscoro(func):
-    """Check if the function is a coroutine or if the function is a ``functools.partial``,
-    check the wrapped function for the same.
-    """
-    if isinstance(func, functools.partial):
-        func = func.func
-    return asyncio.iscoroutinefunction(func)
-
-
 class AccessoryMDNSServiceInfo(ServiceInfo):
     """A mDNS service info representation of an accessory."""
 
@@ -152,7 +138,7 @@ class AccessoryDriver:
         listen_address=None,
         advertised_address=None,
         interface_choice=None,
-        zeroconf_instance=None
+        async_zeroconf_instance=None
     ):
         """
         Initialize a new AccessoryDriver object.
@@ -196,7 +182,7 @@ class AccessoryDriver:
         :param interface_choice: The zeroconf interfaces to listen on.
         :type InterfacesType: [InterfaceChoice.Default, InterfaceChoice.All]
 
-        :param zeroconf_instance: A Zeroconf instance. When running multiple accessories or
+        :param async_zeroconf_instance: An AsyncZeroconf instance. When running multiple accessories or
             bridges a single zeroconf instance can be shared to avoid the overhead
             of processing the same data multiple times.
         """
@@ -220,17 +206,14 @@ class AccessoryDriver:
         self.loop = loop
 
         self.accessory = None
-        if zeroconf_instance is not None:
-            self.advertiser = zeroconf_instance
-        elif interface_choice is not None:
-            self.advertiser = Zeroconf(interfaces=interface_choice)
-        else:
-            self.advertiser = Zeroconf()
+        self.advertiser = async_zeroconf_instance
+        self.interface_choice = interface_choice
+
         self.persist_file = os.path.expanduser(persist_file)
         self.encoder = encoder or AccessoryEncoder()
         self.topics = {}  # topic: set of (address, port) of subscribed clients
         self.loader = loader or Loader()
-        self.aio_stop_event = asyncio.Event(loop=loop)
+        self.aio_stop_event = None
         self.stop_event = threading.Event()
 
         self.safe_mode = False
@@ -255,7 +238,8 @@ class AccessoryDriver:
         """
         try:
             logger.info("Starting the event loop")
-            if threading.current_thread() is threading.main_thread():
+            if threading.current_thread() is threading.main_thread() \
+                    and os.name != "nt":
                 logger.debug("Setting child watcher")
                 watcher = asyncio.SafeChildWatcher()
                 watcher.attach_loop(self.loop)
@@ -265,7 +249,7 @@ class AccessoryDriver:
                     "Not setting a child watcher. Set one if "
                     "subprocesses will be started outside the main thread."
                 )
-            self.add_job(self.start_service)
+            self.add_job(self.async_start())
             self.loop.run_forever()
         except KeyboardInterrupt:
             logger.debug("Got a KeyboardInterrupt, stopping driver")
@@ -276,6 +260,19 @@ class AccessoryDriver:
             logger.info("Closed the event loop")
 
     def start_service(self):
+        """Start the service."""
+        self._validate_start()
+        self.add_job(self.async_start)
+
+    def _validate_start(self):
+        """Validate we can start."""
+        if self.accessory is None:
+            raise ValueError(
+                "You must assign an accessory to the driver, "
+                "before you can start it."
+            )
+
+    async def async_start(self):
         """Starts the accessory.
 
         - Call the accessory's run method.
@@ -287,11 +284,9 @@ class AccessoryDriver:
         All of the above are started in separate threads. Accessory thread is set as
         daemon.
         """
-        if self.accessory is None:
-            raise ValueError(
-                "You must assign an accessory to the driver, "
-                "before you can start it."
-            )
+        self._validate_start()
+        self.aio_stop_event = asyncio.Event()
+
         logger.info(
             "Starting accessory %s on address %s, port %s.",
             self.accessory.display_name,
@@ -301,12 +296,20 @@ class AccessoryDriver:
 
         # Start listening for requests
         logger.debug("Starting server.")
-        self.add_job(self.http_server.async_start, self.loop)
+        await self.http_server.async_start(self.loop)
 
         # Advertise the accessory as a mDNS service.
         logger.debug("Starting mDNS.")
         self.mdns_service_info = AccessoryMDNSServiceInfo(self.accessory, self.state)
-        self.advertiser.register_service(self.mdns_service_info)
+
+        if not self.advertiser:
+            zc_args = {}
+            if self.interface_choice is not None:
+                zc_args["interfaces"] = self.interface_choice
+            self.advertiser = AsyncZeroconf(**zc_args)
+        await self.advertiser.async_register_service(
+            self.mdns_service_info, cooperating_responders=True
+        )
 
         # Print accessory setup message
         if not self.state.paired:
@@ -321,12 +324,15 @@ class AccessoryDriver:
 
     def stop(self):
         """Method to stop pyhap."""
-        self.loop.call_soon_threadsafe(self.loop.create_task, self.async_stop())
+        self.add_job(self.async_stop)
 
     async def async_stop(self):
         """Stops the AccessoryDriver and shutdown all remaining tasks."""
-        await self.async_add_job(self._do_stop)
+        self.stop_event.set()
         logger.debug("Stopping HAP server and event sending")
+        logger.debug("Stopping mDNS advertising for %s", self.accessory.display_name)
+        await self.advertiser.async_unregister_service(self.mdns_service_info)
+        await self.advertiser.async_close()
 
         self.aio_stop_event.set()
 
@@ -338,6 +344,7 @@ class AccessoryDriver:
             self.state.address,
             self.state.port,
         )
+
         await self.async_add_job(self.accessory.stop)
 
         logger.debug(
@@ -352,15 +359,6 @@ class AccessoryDriver:
 
         logger.debug("Stop completed")
 
-    def _do_stop(self):
-        """Stop the mDNS and set the stop event."""
-        logger.debug("Setting stop events, stopping accessory")
-        self.stop_event.set()
-
-        logger.debug("Stopping mDNS advertising for %s", self.accessory.display_name)
-        self.advertiser.unregister_service(self.mdns_service_info)
-        self.advertiser.close()
-
     def add_job(self, target, *args):
         """Add job to executor pool."""
         if target is None:
@@ -374,9 +372,9 @@ class AccessoryDriver:
 
         if asyncio.iscoroutine(target):
             task = self.loop.create_task(target)
-        elif is_callback(target):
+        elif util.is_callback(target):
             self.loop.call_soon(target, *args)
-        elif iscoro(target):
+        elif util.iscoro(target):
             task = self.loop.create_task(target(*args))
         else:
             task = self.loop.run_in_executor(None, target, *args)
@@ -429,6 +427,24 @@ class AccessoryDriver:
         if not subscribed_clients:
             del self.topics[topic]
 
+    def connection_lost(self, client):
+        """Called when a connection is lost to a client.
+
+        This method must be run in the event loop.
+
+        :param client: A client (address, port) tuple that should be unsubscribed.
+        :type client: tuple <str, int>
+        """
+        client_topics = []
+        for topic, subscribed_clients in self.topics.items():
+            if client in subscribed_clients:
+                # Make a copy to avoid changing
+                # self.topics during iteration
+                client_topics.append(topic)
+
+        for topic in client_topics:
+            self.async_subscribe_client_topic(client, topic, subscribe=False)
+
     def publish(self, data, sender_client_addr=None):
         """Publishes an event to the client.
 
@@ -443,18 +459,15 @@ class AccessoryDriver:
         if topic not in self.topics:
             return
 
-        data = {HAP_REPR_CHARS: [data]}
-        bytedata = json.dumps(data).encode()
-
         if threading.current_thread() == self.tid:
-            self.async_send_event(topic, bytedata, sender_client_addr)
+            self.async_send_event(topic, data, sender_client_addr)
             return
 
         self.loop.call_soon_threadsafe(
-            self.async_send_event, topic, bytedata, sender_client_addr
+            self.async_send_event, topic, data, sender_client_addr
         )
 
-    def async_send_event(self, topic, bytedata, sender_client_addr):
+    def async_send_event(self, topic, data, sender_client_addr):
         """Send an event to a client.
 
         Must be called in the event loop
@@ -466,7 +479,7 @@ class AccessoryDriver:
         logger.debug(
             "Send event: topic(%s), data(%s), sender_client_addr(%s)",
             topic,
-            bytedata,
+            data,
             sender_client_addr,
         )
         unsubs = []
@@ -479,7 +492,7 @@ class AccessoryDriver:
                 )
                 continue
             logger.debug("Sending event to client: %s", client_addr)
-            pushed = self.http_server.push_event(bytedata, client_addr)
+            pushed = self.http_server.push_event(data, client_addr)
             if not pushed:
                 logger.debug(
                     "Could not send event to %s, probably stale socket.", client_addr
@@ -498,14 +511,23 @@ class AccessoryDriver:
         to fetch new data.
         """
         self.state.config_version += 1
+        if self.state.config_version > MAX_CONFIG_VERSION:
+            self.state.config_version = 1
         self.persist()
         self.update_advertisement()
 
     def update_advertisement(self):
         """Updates the mDNS service info for the accessory."""
+        self.loop.call_soon_threadsafe(self.async_update_advertisement)
+
+    @callback
+    def async_update_advertisement(self):
+        """Updates the mDNS service info for the accessory from the event loop."""
         logger.debug("Updating mDNS advertisement")
         self.mdns_service_info = AccessoryMDNSServiceInfo(self.accessory, self.state)
-        self.advertiser.update_service(self.mdns_service_info)
+        asyncio.ensure_future(
+            self.advertiser.async_update_service(self.mdns_service_info)
+        )
 
     @callback
     def async_persist(self):
@@ -661,7 +683,7 @@ class AccessoryDriver:
             rep = {
                 HAP_REPR_AID: aid,
                 HAP_REPR_IID: iid,
-                HAP_REPR_STATUS: SERVICE_COMMUNICATION_FAILURE,
+                HAP_REPR_STATUS: HAP_SERVER_STATUS.SERVICE_COMMUNICATION_FAILURE,
             }
 
             try:
@@ -677,7 +699,7 @@ class AccessoryDriver:
 
                 if available:
                     rep[HAP_REPR_VALUE] = char.get_value()
-                    rep[HAP_REPR_STATUS] = CHAR_STAT_OK
+                    rep[HAP_REPR_STATUS] = HAP_SERVER_STATUS.SUCCESS
             except CharacteristicError:
                 logger.error("Error getting value for characteristic %s.", id)
             except Exception:  # pylint: disable=broad-except
@@ -740,10 +762,12 @@ class AccessoryDriver:
                     char.display_name,
                     value,
                 )
-                setter_results[aid][iid] = SERVICE_COMMUNICATION_FAILURE
+                setter_results[aid][
+                    iid
+                ] = HAP_SERVER_STATUS.SERVICE_COMMUNICATION_FAILURE
                 had_error = True
             else:
-                setter_results[aid][iid] = CHAR_STAT_OK
+                setter_results[aid][iid] = HAP_SERVER_STATUS.SUCCESS
 
             # For some services we want to send all the char value
             # changes at once.  This resolves an issue where we send
@@ -777,10 +801,10 @@ class AccessoryDriver:
                         client_addr,
                         service_name,
                     )
-                    set_result = SERVICE_COMMUNICATION_FAILURE
+                    set_result = HAP_SERVER_STATUS.SERVICE_COMMUNICATION_FAILURE
                     had_error = True
                 else:
-                    set_result = CHAR_STAT_OK
+                    set_result = HAP_SERVER_STATUS.SUCCESS
 
                 for iid in service_data[SERVICE_IIDS]:
                     setter_results[aid][iid] = set_result
