@@ -1,17 +1,39 @@
 import asyncio
 import base64
-import socket
+import functools
+import json
 import random
-import binascii
-import sys
+import socket
+from uuid import UUID
 
+from .const import BASE_UUID
 
-ALPHANUM = '0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZ'
-HEX_DIGITS = '0123456789ABCDEF'
+ALPHANUM = "0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZ"
+HEX_DIGITS = "0123456789ABCDEF"
 
 rand = random.SystemRandom()
 
 
+def callback(func):
+    """Decorator for non blocking functions."""
+    setattr(func, "_pyhap_callback", True)
+    return func
+
+
+def is_callback(func):
+    """Check if function is callback."""
+    return "_pyhap_callback" in getattr(func, "__dict__", {})
+
+
+def iscoro(func):
+    """Check if the function is a coroutine or if the function is a ``functools.partial``,
+    check the wrapped function for the same.
+    """
+    if isinstance(func, functools.partial):
+        func = func.func
+    return asyncio.iscoroutinefunction(func)
+
+
 def get_local_address():
     """
     Grabs the local IP address using a socket.
@@ -59,7 +81,8 @@ def generate_mac():
     :rtype: str
     """
     return "{}{}:{}{}:{}{}:{}{}:{}{}:{}{}".format(
-        *(rand.choice(HEX_DIGITS) for _ in range(12)))
+        *(rand.choice(HEX_DIGITS) for _ in range(12))
+    )
 
 
 def generate_setup_id():
@@ -71,10 +94,7 @@ def generate_setup_id():
     :return: 4 digit alphanumeric code.
     :rtype: str
     """
-    return ''.join([
-        rand.choice(ALPHANUM)
-        for i in range(4)
-    ])
+    return "".join([rand.choice(ALPHANUM) for i in range(4)])
 
 
 def generate_pincode():
@@ -84,53 +104,24 @@ def generate_pincode():
     :return: pincode in format ``xxx-xx-xxx``
     :rtype: bytearray
     """
-    return '{}{}{}-{}{}-{}{}{}'.format(
-        *(rand.randint(0, 9) for i in range(8))
-    ).encode('ascii')
-
-
-def b2hex(bts):
-    """Produce a hex string representation of the given bytes.
-
-    :param bts: bytes to convert to hex.
-    :type bts: bytes
-    :rtype: str
-    """
-    return binascii.hexlify(bts).decode("ascii")
-
-
-def hex2b(hex_str):
-    """Produce bytes from the given hex string representation.
-
-    :param hex: hex string
-    :type hex: str
-    :rtype: bytes
-    """
-    return binascii.unhexlify(hex_str.encode("ascii"))
-
-
-tohex = bytes.hex if sys.version_info >= (3, 5) else b2hex
-"""Python-version-agnostic tohex function. Equivalent to bytes.hex in python 3.5+.
-"""
-
-fromhex = bytes.fromhex if sys.version_info >= (3, 5) else hex2b
-"""Python-version-agnostic fromhex function. Equivalent to bytes.fromhex in python 3.5+.
-"""
+    return "{}{}{}-{}{}-{}{}{}".format(*(rand.randint(0, 9) for i in range(8))).encode(
+        "ascii"
+    )
 
 
 def to_base64_str(bytes_input) -> str:
-    return base64.b64encode(bytes_input).decode('utf-8')
+    return base64.b64encode(bytes_input).decode("utf-8")
 
 
 def base64_to_bytes(str_input) -> bytes:
-    return base64.b64decode(str_input.encode('utf-8'))
+    return base64.b64decode(str_input.encode("utf-8"))
 
 
 def byte_bool(boolv):
-    return b'\x01' if boolv else b'\x00'
+    return b"\x01" if boolv else b"\x00"
 
 
-async def event_wait(event, timeout, loop=None):
+async def event_wait(event, timeout):
     """Wait for the given event to be set or for the timeout to expire.
 
     :param event: The event to wait for.
@@ -143,7 +134,27 @@ async def event_wait(event, timeout, loop=None):
     :rtype: bool
     """
     try:
-        await asyncio.wait_for(event.wait(), timeout, loop=loop)
+        await asyncio.wait_for(event.wait(), timeout)
     except asyncio.TimeoutError:
         pass
     return event.is_set()
+
+
+def uuid_to_hap_type(uuid):
+    """Convert a UUID to a HAP type."""
+    long_type = str(uuid).upper()
+    if not long_type.endswith(BASE_UUID):
+        return long_type
+    return long_type.split("-", 1)[0].lstrip("0")
+
+
+def hap_type_to_uuid(hap_type):
+    """Convert a HAP type to a UUID."""
+    if "-" in hap_type:
+        return UUID(hap_type)
+    return UUID("0" * (8 - len(hap_type)) + hap_type + BASE_UUID)
+
+
+def to_hap_json(dump_obj):
+    """Convert an object to HAP json."""
+    return json.dumps(dump_obj, separators=(",", ":")).encode("utf-8")
